/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import ru.ulstu.extractor.repository.BranchRepository;
import ru.ulstu.extractor.repository.RepositoryRepository;
import springfox.documentation.annotations.ApiIgnore;

import static ru.ulstu.extractor.controller.Route.DELETE_BRANCH;
import static ru.ulstu.extractor.controller.Route.LIST_REPOSITORY_BRANCHES;

@Controller
@ApiIgnore
public class BranchController {
    private final RepositoryRepository repositoryRepository;
    private final BranchRepository branchRepository;

    public BranchController(RepositoryRepository repositoryRepository, BranchRepository branchRepository) {
        this.repositoryRepository = repositoryRepository;
        this.branchRepository = branchRepository;
    }

    @GetMapping(LIST_REPOSITORY_BRANCHES)
    public String indexBranch(
            Model model,
            @RequestParam int repositoryId) {
        model.addAttribute("branches", branchRepository.findByRepositoryId(repositoryId));
        model.addAttribute("repository", repositoryRepository.findById(repositoryId).get());
        return LIST_REPOSITORY_BRANCHES;
    }

    @GetMapping(DELETE_BRANCH)
    public String deleteBranch(Model model,
                               @RequestParam int repositoryId,
                               @RequestParam Integer id) {
        branchRepository.deleteById(id);
        model.addAttribute("branches", branchRepository.findByRepositoryId(repositoryId));
        model.addAttribute("repository", repositoryRepository.findById(repositoryId).get());
        return LIST_REPOSITORY_BRANCHES;
    }
}
