/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import ru.ulstu.extractor.repository.CommitRepository;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.stream.Collectors;

import static ru.ulstu.extractor.controller.Route.STATISTIC;

@Controller
@ApiIgnore
public class StatisticController {
    private final CommitRepository commitRepository;

    public StatisticController(CommitRepository commitRepository) {
        this.commitRepository = commitRepository;
    }

    @GetMapping(STATISTIC)
    public String indexBranch(Model model) {
        model.addAttribute("commitAuthorData", authorCommits());
        model.addAttribute("commitUrlData", urlCommits());
        model.addAttribute("commitTimeData", timeCommits());
        String[] date = new String[timeCommits().size()];
        for (int i = 0; i < timeCommits().size(); i++) {
            date[i] = timeCommits().get(i)[0].toString();
        }
        model.addAttribute("dates", date);
        String[] url = new String[urlCommits().size()];
        for (int i = 0; i < urlCommits().size(); i++) {
            url[i] = urlCommits().get(i)[0].toString().substring(urlCommits().get(i)[0].toString().lastIndexOf("/") + 1);
        }
        model.addAttribute("urls", url);
        model.addAttribute("commitEntityData", entityCommits());
        model.addAttribute("commitTimeEntityData", allTimeEntityCommits());
        return STATISTIC;
    }

    public List<Object[]> authorCommits() {
        List<Object[]> authorCommits = commitRepository.getCommitAuthorStatistic().stream()
                .map(stat -> new Object[]{stat.getAuthor(), stat.getCountCommit()})
                .collect(Collectors.toList());
        return authorCommits;
    }

    public List<Object[]> urlCommits() {
        List<Object[]> urlCommits = commitRepository.getCommitUrlStatistic().stream()
                .map(stat -> new Object[]{stat.getUrl(), stat.getCountCommit()})
                .collect(Collectors.toList());
        return urlCommits;
    }

    public List<Object[]> timeCommits() {
        List<Object[]> timeCommits = commitRepository.getCommitTimeStatistic().stream()
                .map(stat -> new Object[]{stat.getDate(), stat.getCountCommit()})
                .collect(Collectors.toList());
        return timeCommits;
    }

    public List<Object[]> entityCommits() {
        List<Object[]> entityCommits = commitRepository.getCommitEntityStatistic().stream()
                .map(stat -> new Object[]{stat.getEntity(), stat.getCountCommit()})
                .collect(Collectors.toList());
        for (int i = 0; i < entityCommits.size(); i++) {
            entityCommits.get(i)[0] = Boolean.TRUE.equals(entityCommits.get(i)[0])
                    ? "Есть сущности"
                    : (entityCommits.get(i)[0] == null
                    ? "Нет данных"
                    : "Нет сущностей");
        }
        return entityCommits;
    }

    public List<Object[]> timeEntityCommits() {
        List<Object[]> timeEntityCommits = commitRepository.getCommitTimeEntityStatistic().stream()
                .map(stat -> new Object[]{stat.getDate(), stat.getCountCommit()})
                .collect(Collectors.toList());
        return timeEntityCommits;
    }

    public List<Object[]> allTimeEntityCommits() {
        List<Object[]> allTimeEntityCommits = commitRepository.getCommitTimeStatistic().stream()
                .map(stat -> new Object[]{stat.getDate(), stat.getCountCommit()})
                .collect(Collectors.toList());
        int j = 0;
        for (int i = 0; i < allTimeEntityCommits.size(); i++) {
            if ((j < timeEntityCommits().size()) && (allTimeEntityCommits.get(i)[0].equals(timeEntityCommits().get(j)[0]))) {
                allTimeEntityCommits.get(i)[1] = timeEntityCommits().get(j)[1];
                j++;
            } else {
                if (i > 0) {
                    allTimeEntityCommits.get(i)[1] = allTimeEntityCommits.get(i - 1)[1];
                }
            }
        }
        return allTimeEntityCommits;
    }
}
